

-- Level Class --
-----------------
local Level = {}

-- Dependencies --
------------------
local Pad = require("source.Pad")
local Body = require("source.Body")
local GravityBall = require("source.GravityBall")
  
-- Declarations --
------------------
local map
local arrow
local lastStart, lastDestination
local soundComplete = love.audio.newSource("assets/complete.ogg", "static")
local completePlayed = false



function Level:getPad(name)
  for i, pad in ipairs(self.pads) do
    if pad.id == name then return pad end
  end
end


function Level:getPassenger(id)
  for i, passenger in ipairs(self.passengers) do
    if passenger.id == id then return passenger end
  end
end


function Level:complete()
  if not self.completed then
    soundComplete:play()
    self.completed = true
  end
end


function Level:addBody(image, x, y, xvel, yvel)
  local body = Body:new(image, x, y, xvel, yvel)
  table.insert(self.bodies, body)
end


function Level:spawnPassenger()
  local Passenger = require("source.Passenger")
  local randomStart = math.random(1, #self.pads-1)
  local randomDestination = math.random(1, #self.pads-1)
  if randomStart == randomDestination or randomStart == lastStart or randomDestination == lastDestination then
    self:spawnPassenger()
    return
  end
  lastStart = randomStart
  lastDestination = randomDestination
  
  local passenger = Passenger:new(#self.passengers+1, self:getPad(tostring(randomStart)), self:getPad(tostring(randomDestination)), math.random(15, 20))
  self.passengers[#self.passengers+1] = passenger
  self.lastPassenger = passenger.id
  self.passengerAmount = self.passengerAmount-1
  return passenger.id
end


function Level:init()
  self.gravity = 50
  
  -- Level Completed Arrow
  arrow = {}
  arrow.image = love.graphics.newImage("assets/arrow.png")
  arrow.w = arrow.image:getWidth()
  arrow.h = arrow.image:getHeight()
  arrow.x = WIDTH / 2 - arrow.w / 2
  arrow.y = 8
  arrow.timer = 0
end



function Level:setLevel(level)
  self.completed = false
  self.pads = {}
  self.passengers = {}
  self.gravityBalls = {}
  self.mapCollisionTiles = {}
  self.exitCollisionTiles = {}
  self.bodies = {}
  self.map = sti.new("assets/level"..level)
  self.passengerAmount = self.map.properties.passengerAmount
  map = self.map
  
  
  -- Map collisionMap --
  local collision = map:getCollisionMap("level")
  for y = 1, #collision.data do
    for x = 1, #collision.data[1] do
    local tile = {id = collision.data[y][x], x = x * 8 - 8, y = y * 8 - 7, w = 8, h = 8-2}
    if tile.id == 1 then table.insert(self.mapCollisionTiles, tile) end
    end
  end
  
  -- Exit collisionMap --
  collision = map:getCollisionMap("exit")
  for y = 1, #collision.data do
    for x = 1, #collision.data[1] do
    local tile = {id = collision.data[y][x], x = x * 8 - 8, y = y * 8 - 7, w = 8, h = 8-2}
    if tile.id == 1 then table.insert(self.exitCollisionTiles, tile) end
    end
  end
  
  -- Place pads --
  ----------------
  for y = 1, map.layers["objects"].height do
    for x = 1, map.layers["objects"].width do
      if map.layers["objects"].data[y][x] then
        local id = map.layers["objects"].data[y][x].gid - map.tilesets[map.layers["objects"].data[y][x].tileset].firstgid
        local padid
        for i, tile in ipairs(map.tilesets[map.layers["objects"].data[y][x].tileset].tiles) do
          if tile.id == id then
            padid = tile.properties["padid"]
          end
        end
        self.pads[#self.pads+1] = Pad:new(padid, x*8-8+2, y*8, 48-4, 1)
      end
    end
  end
  
  -- Place Gravity Balls --
  -------------------------
  if map.layers["gravity"] then
  for y = 1, map.layers["gravity"].height do
    for x = 1, map.layers["gravity"].width do
      if map.layers["gravity"].data[y][x] then
        local tile = map.layers["gravity"].data[y][x]
        local id = map.layers["gravity"].data[y][x].gid - map.tilesets[map.layers["gravity"].data[y][x].tileset].firstgid
        for i, tile in ipairs(map.tilesets[map.layers["gravity"].data[y][x].tileset].tiles) do
          if tile.id == id then
            self.gravityBalls[#self.gravityBalls+1] = GravityBall:new(x*8-8, y*8-8, tile.properties["direction"], map.layers["gravity"].properties["speed"])
          end
        end
        
      end
    end
  end
  end
  
  return map
end
  
  
function Level:checkCollisions(dt)
  -- Map --
  for i, tile in ipairs(self.mapCollisionTiles) do
    for i, hitbox in ipairs(player.hitboxes) do
      if boxCollision(hitbox, tile) then
        player:kill()
        return
      end
    end
  end
  
  -- Exit --
  if not map.layers["exit"].visible then return end
  for i, tile in ipairs(self.exitCollisionTiles) do
    for i, hitbox in ipairs(player.hitboxes) do
      if boxCollision(hitbox, tile) then
        player:kill()
        return
      end
    end
  end
end


function Level:update(dt)
  -- Spawn passengers --
  if #self.passengers == 0 and tonumber(self.passengerAmount) > 0 then
    self:spawnPassenger()
    if tonumber(map.properties.passengerAmount) >= 4 and self.passengerAmount >= 2 and math.random(1, 3) == 3 then
      self:spawnPassenger()
    end
  end
  
  if not player.removed then
    -- Pads --
    for i, pad in ipairs(self.pads) do pad:update(dt) end
    -- Check collisions --
    self:checkCollisions(dt)
    -- Update STI map --
    map:update(dt)
  end

  -- Update body parts --
  for i, body in ipairs(self.bodies) do
    if not body.removed then body:update(dt)
    else table.remove(self.bodies, i) end
  end
  
  -- Level completed --
  if #self.passengers == 0 and self.passengerAmount == 0 then
    self:complete()
    map.layers["exit"].visible = false
    if not player.hidden and player.y < -player.h + 2 and player.x > WIDTH / 2 - 24 and player.x < WIDTH / 2 + 24 then map.completed = true end
  end
end


function Level:draw()
  -- Draw Bodies --
  for i, body in ipairs(self.bodies) do body:draw() end
  
  -- Draw Arrow --
  local dt = love.timer.getDelta()
  arrow.timer = arrow.timer + dt
  arrow.y = arrow.y + math.sin(arrow.timer * 8) * 20 * dt
  if self.completed then love.graphics.draw(arrow.image, arrow.x, arrow.y) end
end

return Level
