

-- Passenger Class --
---------------------
local Object = require("source.Object")
local Passenger = Object:new()
Passenger.__index = Passenger

local Level = require("source.Level")

local sounds = {
  ["pad1"] = love.audio.newSource("assets/pad1.ogg", "static"),
  ["pad2"] = love.audio.newSource("assets/pad2.ogg", "static"),
  ["pad3"] = love.audio.newSource("assets/pad3.ogg", "static"),
  ["pad4"] = love.audio.newSource("assets/pad4.ogg", "static"),
  ["pad5"] = love.audio.newSource("assets/pad5.ogg", "static"),
  ["pad6"] = love.audio.newSource("assets/pad6.ogg", "static"),
  ["pad7"] = love.audio.newSource("assets/pad7.ogg", "static"),
  ["pad8"] = love.audio.newSource("assets/pad8.ogg", "static"),
  ["pad9"] = love.audio.newSource("assets/pad9.ogg", "static"),
  ["pad10"] = love.audio.newSource("assets/pad10.ogg", "static"),
  ["hey1"] = love.audio.newSource("assets/whistle.ogg", "static"),
  ["hey2"] = love.audio.newSource("assets/whistle.ogg", "static"),
  ["thanks1"] = love.audio.newSource("assets/thanks.ogg", "static"),
  ["thanks2"] = love.audio.newSource("assets/thankyou.ogg", "static"),
  }


function Passenger:new(id, start, destination, money)
  local self = setmetatable({}, Passenger)
  self.id = id
  self.start = start
  self.destination = destination
  self.money = money
  self.position = start.id
  self.w = 10
  self.h = 14
  self.speed = 15
  self.x = math.random(start.x, start.x + start.w - self.w)
  self.y = start.y
  self.arrived = false
  self.color = {math.random(200), math.random(200), 0}
  self.mode = "waiting"
  self.timer = 0
  self.spawnTimer = 0
  self.spawnTime = math.random(0, 2)
  self.waitTime = 30
  self.whistlePlayed = false
  self.thanksPlayed = false
  self.targety = self.y - self.h
  self.waveAnim = newAnimation(love.graphics.newImage("assets/passengerAnim.png"), self.w, self.h, .1, 0)
  self.waveAnim:setMode("bounce")
  self.waveAnim:seek(math.random(1, #self.waveAnim.frames))
  self.waveAnim:play()
  self.moveAnim = newAnimation(love.graphics.newImage("assets/passengerMove.png"), self.w, self.h, .1, 0)
  self.moveAnim:play()
  self.removed = false
  self.pitch = math.random(-100, 600)/ 100 / 12
  return self
end


function Passenger:getInTaxi()
  player.passengerId = self.id
  self.timer = nil
  self.position = "air"
  sounds["pad"..self.destination.id]:setPitch(1+self.pitch)
  sounds["pad"..self.destination.id]:play()
end


function Passenger:move(xvel, yvel)
  local dt = love.timer.getDelta()
  if xvel then
    if xvel > 0 then
      self.mode = "right"
    elseif xvel < 0 then
      self.mode = "left"
    end
    self.x = self.x + xvel * dt
  end
  if yvel ~= 0 then
    self.mode = "waiting"
    self.y = self.y + yvel * dt
  end
end


function Passenger:update(dt)
  
  if self.timer then self.timer = self.timer + dt end

  self.mode = "waiting"
  self.waveAnim:update(dt)
  self.moveAnim:update(dt)
  self.spawnTimer = self.spawnTimer + dt

  -- Decrease money over time
  if self.money > 0 then
    self.money = self.money - .25 * dt
  else
    self.money = 0
  end

  -- despawn passenger if he waits too long
  if self.timer and self.timer > self.waitTime then
    if self.y < self.targety + self.h then self:move(0, self.speed) end
    if self.y > self.targety + self.h then
      Level.passengerAmount = Level.passengerAmount + 1
      self:destroy()
    end
  end
    
  -- Arrived at destination pad
  if player.padId == self.destination.id and self.position ~= self.start.id and not self.arrived then
    self.position = self.destination.id
    self.x = player.x + player.w / 2 - self.w / 2
    self.y = self.destination.y - self.h
    self.targety = self.y
    player.passengerId = nil
    player.money = player.money + math.floor(self.money)
    self.arrived = true
  end
  
  -- Movement --
  --------------
  if self.position == self.start.id then
    -- Move up
    if self.spawnTimer > self.spawnTime then
      if not self.whistlePlayed then
        local rnd = math.random(1, 2)
        sounds["hey"..rnd]:setPitch(1+self.pitch)
        sounds["hey"..rnd]:play()
        self.whistlePlayed = true
      end
      if self.y > self.targety and self.timer < self.waitTime then
        self:move(0, -self.speed)
        end
      if self.y < self.targety then self.y = self.targety end
    end
    
    self.waveAnim:play()
    
    -- Go to taxi and get in
    if self.position == player.padId and self.y == self.targety and not player.passengerId then
      self.waveAnim:stop()
      self.waveAnim:seek(8)
      if self.x + self.w / 2 < player.x + player.w / 2 then self:move(self.speed, 0)
        if  self.x + self.w / 2 > player.x + player.w / 2 then self:getInTaxi() end
      elseif self.x + self.w / 2 > player.x + player.w / 2 then self:move(-self.speed, 0)
        if  self.x + self.w / 2 < player.x + player.w / 2 then self:getInTaxi() end
      end
    end
    
  -- Get out
  elseif self.position == self.destination.id then
    
    -- Thanks sound played ?
    if not self.thanksPlayed then
      local rnd = math.random(1, 4)
      if rnd < 3 then
        sounds["thanks"..rnd]:setPitch(1+self.pitch)
        sounds["thanks"..rnd]:play()
      end
    self.thanksPlayed = true
    end
  
    
    if self.x + self.w / 2 < self.destination.x + self.destination.w + 8 then self:move(self.speed, 0) end
    if self.x + self.w / 2 > self.destination.x + self.destination.w - self.w / 2 then
      self.x = self.destination.x + self.destination.w - self.w
      if self.y < self.targety + self.h then
        self:move(0, self.speed)
      end
      if self.y > self.targety + self.h then
        if player.passengerId == self.id then player.passengerId = nil end
        self:destroy()
      end
    end
  end
end


function Passenger:draw()
  love.graphics.setScissor(self.x*SCALEX, self.targety*SCALEY, self.w*SCALEX, self.h*SCALEY)
  if self.position ~= "air" then
    if self.mode == "waiting" then self.waveAnim:draw(self.x, self.y) end
    if self.mode == "right" then self.moveAnim:draw(self.x, self.y, 0, -1, 1, self.w)
    elseif self.mode == "left" then self.moveAnim:draw(self.x, self.y) end
  end
  love.graphics.setScissor()
end

return Passenger
