
-- // Taxi \\ --
----------------
local Object = require("source.Object")
local Taxi = Object:new()
Taxi.__index = Taxi

-- Dependencies --
------------------
local SceneManager = require("source.SceneManager")
local Fader = require("source.Fader")
local Level = require("source.Level")

-- Declarations --
------------------
local keyboard = love.keyboard
local joystick = love.joystick
local passengerBody = love.graphics.newImage("assets/passengerBody.png")
local taxiBody = {
  love.graphics.newImage("assets/taxiBody1.png"),
  love.graphics.newImage("assets/taxiBody2.png"),
  love.graphics.newImage("assets/taxiBody3.png"),
}

local left, right, up, down
local sounds = {
  thrust = love.audio.newSource("assets/thrust.ogg", "static"),
  explosion = love.audio.newSource("assets/explosion.ogg", "static"),
  fuel = love.audio.newSource("assets/nofuel.ogg", "static"),
  noo = love.audio.newSource("assets/noo.ogg", "static")
}

function Taxi:new(x, y)
  local self = setmetatable({}, Taxi)
  self.image = love.graphics.newImage("assets/taxi.png")
  self.x = x or 0
  self.y = y or 0
  self.w = self.image:getWidth()
  self.h = self.image:getHeight()
  self.accX = 75
  self.accY = 130
  self.state = "flying"
  self.direction = "left"
  self.money = 0
  self.lives = 5
  self.gravity = Level.gravity
  self.velX = 0
  self.velY = 0
  self.fuel = 100
  self.padId = ""
  self.consumption = 2.75
  self.floor = HEIGHT
  self.hitboxes = {}
  self.hidden = false
  -- self.passengerId -- gets set in Level lua
  
  self.flameImage = love.graphics.newImage("assets/flame.png")
  local iw, ih = self.flameImage:getWidth(), self.flameImage:getHeight()
  self.flames = {}
  self.flames[1] = love.graphics.newQuad(0, 0, 5, 5, iw, ih)
  self.flames[2] = love.graphics.newQuad(5, 0, 5, 5, iw, ih)
  self.flames[3] = love.graphics.newQuad(10, 0, 5, 5, iw, ih)
  self.flames[4] = love.graphics.newQuad(15, 0, 5, 5, iw, ih)

  self.gear = {}
  self.gear.image = love.graphics.newImage("assets/landinggear.png")
  self.gear.w = self.gear.image:getWidth()
  self.gear.h = self.gear.image:getHeight()
  self.gear.x = self.x + self.w/2 - self.gear.w/2
  self.gear.y = self.y + self.h
  self.gear.lowered = true
  
  return self
end


function Taxi:getHitbox(index)
  return self.hitboxes[index]
end


function Taxi:setFloor(v)
  self.floor = v
end


function Taxi:setConsumption(v)
  self.consumption = v 
end


function Taxi:hide()
  self.hidden = true
end


function Taxi:setPosition(x, y)
  self.x = x
  self.y = y
end


function Taxi:respawn(x, y)
  self:setPosition(x, y)
  self.velX = 0
  self.velY = 0
  self.gear.lowered = true
  self.hidden = false
  self.passengerId = nil
end


function Taxi:kill()
  self.lives = self.lives - 1
  
  -- Passenger --
  ---------------
  if self.passengerId then
    Level.passengerAmount = Level.passengerAmount + 1
    Level:spawnPassenger()
    local pitch = Level:getPassenger(self.passengerId).pitch
    sounds.noo:setPitch(1+pitch)
    sounds.noo:play()
    Level:addBody(passengerBody, self.x + self.w / 2, self.y + self.h / 2, self.velX + math.random(-30, 30), self.velY - 60)
    Level:getPassenger(self.passengerId):destroy()
  end
  
  self.passengerId = nil
  
  -- Taxi --
  ----------
  Level:addBody(taxiBody[1], self.x, self.y + self.h / 2, math.random(-50, 50), -50)
  Level:addBody(taxiBody[2], self.x + self.w / 2, self.y + self.h / 2, math.random(-50, 50), -60)
  Level:addBody(taxiBody[3], self.x + self.w, self.y + self.h / 2, math.random(-50, 50), -40)
  self.fuel = 100
  self.money = 0
  sounds.thrust:stop()
  sounds.explosion:stop()
  sounds.explosion:play()
  
  local gasstation = Level:getPad("Gas Station")
  self:respawn(gasstation.x + gasstation.w / 2 - 16, gasstation.y - 10)
  
  if self.lives < 1 then 
    self:destroy()
    Fader.fadeTo("GameScene") -- Replace by Game Over Screen
    SceneManager.getArray("GameScene").level = 1
  end

end
 

function Taxi:move(xvel, yvel)
  local dt = love.timer.getDelta()
  if xvel ~= 0 and self.velX > -150 and self.velX < 150 then self.velX = self.velX + xvel * dt end
  if yvel ~= 0 and self.velY > -150 and self.velY < 150 then self.velY = self.velY + yvel * dt end
  self.fuel = self.fuel - self.consumption * dt
  self.padId = ""
  if xvel > 0 then self.direction = "left"
  elseif xvel < 0 then self.direction = "right" end
end


function Taxi:updateHitboxes(dt)
  -- Update hitboxes
  self.hitboxes[1] = {x = self.x + 2, y = self.y, w = self.w / 2, h = self.h / 2}
  self.hitboxes[2] = {x = self.x, y = self.y + self.h / 2, w = self.w, h = self.h / 2}
  if self.direction == "right" then self.hitboxes[1].x = self.x + 12 end
  if self.gear.lowered then self.hitboxes[2].h = self.h / 2 + self.gear.h end
end


function Taxi:updateMovement(dt)
  -- Bind Controller --
  if CURRENT_INPUT_DEVICE == "joystick" and #CONTROLLER > 0 then
    local controller = CONTROLLER[1]
    local ax, ay = CONTROLLER[1]:getAxis(1), CONTROLLER[1]:getAxis(2)
    local stickup, stickdown, stickleft, stickright = ay < -.3, ay > .3, ax < -.5, ax > .5
    left = stickleft or controller:isDown(3)
    right = stickright or controller:isDown(4)
    up = stickup or controller:isDown(1)
    down = stickdown or controller:isDown(2)
    
  -- Bind Keyboard --
  elseif CURRENT_INPUT_DEVICE == "keyboard" then 
    left = keyboard.isDown("left")
    right = keyboard.isDown("right")
    up = keyboard.isDown("up")
    down = keyboard.isDown("down")
  end
  
  -- Apply gravity --
  if self.velY < 150 then self.velY = self.velY + self.gravity * dt end
 
  -- Apply velocities --
  self.x = self.x + self.velX * dt
  self.y = self.y + self.velY * dt
  
  -- Apply movement input --
  if self.fuel > 0 then
    if not self.gear.lowered then
      if left then self:move(-self.accX, 0) end
      if right then self:move(self.accX, 0) end
    end
    if up then self:move(0, -self.accY) end
    if down then self:move(0, self.accY/2) end 
  end
end


function Taxi:updateFuel(dt)
  -- Refill gas tank and pay money
  if self.padId == "Gas Station" then
    if self.fuel < 100 and self.money > 0 then
      self.fuel = self.fuel + 10 * dt
      self.money = self.money - 5 * dt
      if self.money < 0 then self.money = 0 end
      if self.fuel > 100 then self.fuel = 100 end
    end
  end
  
  -- Play sound if fuel is near empty
  if self.fuel < 20 and not sounds.fuel:isPlaying() then
    sounds.fuel:play()
  end
  
  -- Play thrust sound
  if self.fuel > 0 and self.state == "flying" then
    sounds.thrust:pause()
    if up or down then
        sounds.thrust:play()
    elseif left and not self.gear.lowered or right and not self.gear.lowered then
        sounds.thrust:play()
    end
  else sounds.thrust:stop() end
    
end


function Taxi:update(dt)
  
  if not self.hidden then 
    -- Update gear --
    self.gear.x = self.x + self.w/2 - self.gear.w/2
    self.gear.y = self.y + self.h
    if self.gear.lowered then
      if self.velX > 0 then self.velX = self.velX - 15 * dt
      elseif self.velX < 0 then self.velX = self.velX + 15 * dt end
    end
  
    -- Update stuff --
    self:updateHitboxes(dt)
    self:updateMovement(dt)
    self:updateFuel(dt)
  
    -- Teleport --
    if self.x > WIDTH then self.x = -self.w end
    if self.x < -self.w then self.x = WIDTH end
    if self.y > HEIGHT - self.h/2 - 16 then self.y = -self.h end
    if self.y < -self.h then self.y = HEIGHT - self.h/2 - 16 end
    
    -- Setup boundaries --
    if self.y + self.h + self.gear.h >= self.floor and not up then
      self.y = self.floor - (self.h + self.gear.h)
      self.state = "landed"
      sounds.thrust:stop()
      self.velX = 0
      if not up then self.velY = 0 end
    else
      self.state = "flying"
    end
  
    -- Default floor height
    self.floor = HEIGHT
  end

end


function Taxi:draw()
  if not self.hidden then
  -- Flip Taxi --
  if self.direction == "right" then love.graphics.draw(self.image, self.x, self.y, 0, -1, 1, self.w, 0)
  else love.graphics.draw(self.image, self.x, self.y) end

  -- Draw Gear --
  if self.gear.lowered then love.graphics.draw(self.gear.image, self.gear.x, self.gear.y) end
  
  -- Draw Flames --
  if self.fuel <= 0 then return end
  if self.state == "flying" then
    if right and not self.gear.lowered then love.graphics.draw(self.flameImage, self.flames[1], self.x-5, self.y + self.h / 2-2) end
    if left and not self.gear.lowered then love.graphics.draw(self.flameImage, self.flames[2], self.x + self.w, self.y + self.h / 2-2) end
    if down then
      if self.direction == "right" then
        love.graphics.draw(self.flameImage, self.flames[4], self.x + self.w / 2 + 2, self.y-5)
      else
        love.graphics.draw(self.flameImage, self.flames[4], self.x + self.w / 2 - 4, self.y-5)
      end
    end
    if up then love.graphics.draw(self.flameImage, self.flames[3], self.x + self.w / 2, self.y + self.h) end
  end
end
end


function Taxi:keypressed(key)
  if key == " " then self.gear.lowered = not self.gear.lowered end
end


function Taxi:joystickpressed(joystick, button)
  if CONTROLLER[1]:isGamepadDown("a") then self.gear.lowered = not self.gear.lowered end
end

return Taxi
